﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;


namespace BitEffectCS
{
    public class FastBitmap
    {
        Color[] table = new Color[256];

        public void InitGamma(double gamma)
        {
            for (int i = 0; i < 256; i++)
            {
                int val = (int)(255 * Math.Pow((double)i / 255, 1 / gamma));
                if (val > 255)
                {
                    table[i] = Color.FromArgb(255, 255, 255);
                }
                else if (val < 0)
                {
                    table[i] = Color.FromArgb(0, 0, 0);
                }
                else
                {
                    table[i] = Color.FromArgb(val, val, val);
                }
            }
        }

        public void GammaChange(BitmapData img, int xSize, int ySize)
        {
            var outdata = new byte[xSize, ySize];
            for (int i = 0; i < ySize; i++)
            {
                for (int j = 0; j < xSize; j++)
                {
                    var col = GetPixel(img, j, i);
                    SetPixel(img, j, i, table[col.B]);
                }
            }
        }

        public BitmapData BeginAccess(Bitmap bmp)
        {
            return bmp.LockBits(new Rectangle(0, 0, bmp.Width, bmp.Height), ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
        }

        public Color GetPixel(BitmapData img, int x, int y)
        {
            unsafe
            {
                var adr = (byte*)img.Scan0;
                int pos = x * 3 + img.Stride * y;
                byte b = adr[pos + 0];
                byte g = adr[pos + 1];
                byte r = adr[pos + 2];
                return Color.FromArgb(r, g, b);
            }
        }

        public void SetPixel(BitmapData img, int x, int y, Color col)
        {
            unsafe{
                var adr = (byte*)img.Scan0;
                int pos = x * 3 + img.Stride * y;
                adr[pos + 0] = col.B;
                adr[pos + 1] = col.G;
                adr[pos + 2] = col.R;
            }
        }

        public void EndAccess(Bitmap bitmap, BitmapData img)
        {
            bitmap.UnlockBits(img);
            bitmap = null;
        }

    }
}
