﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;

//  ちなみにSonyReaderでの最適解像度は(584, 754)


namespace S1500PDFConverter
{
    public partial class MainForm : Form
    {
        private int numOfPages = 0;
        private int currentPage = 1;
        private System.Drawing.Rectangle leftRect = new System.Drawing.Rectangle();
        private System.Drawing.Rectangle rightRect = new System.Drawing.Rectangle();

        private System.Drawing.Image leftBmp = null;
        private System.Drawing.Image rightBmp = null;

        private delegate void MessageDelegate(string str);
        private delegate void EndDelegate();

        public MainForm()
        {
            InitializeComponent();
            updateImages();
        }


        private void SetSource_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "PDFファイル(*.pdf)|*.pdf";
            ofd.Title = "変換元PDFファイルを選択してください";
            ofd.RestoreDirectory = true;
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                filePath.Text = ofd.FileName;
            }
        }

        private void CreatePdfMsg(string str)
        {
            currentPage++;
            Page.Text = currentPage + " / " + numOfPages;
        }

        private void CreatePdfEndOperation()
        {
        }

        private void CreatePdfData_Click(object sender, EventArgs e)
        {
            int sizeX = Int32.Parse(TargetScreenWidth.Text);
            int sizeY = Int32.Parse(TargetScreenHeight.Text);

            System.Drawing.Size pageSize;
            if (KeepAspectRatio.Checked)
            {
                float ScreenAspect = (float)sizeX / (float)sizeY;
                float DocumentAspect = (float)leftRect.Width / (float)leftRect.Height;
                if (ScreenAspect > DocumentAspect)
                {
                    pageSize = new System.Drawing.Size((int)((float)sizeY * DocumentAspect), sizeY);
                }
                else
                {
                    pageSize = new System.Drawing.Size(sizeX, (int)((float)sizeX / DocumentAspect));
                }
            }
            else
            {
                pageSize = new System.Drawing.Size(sizeX, sizeY);
            }

            String fileName = Path.GetFileName(filePath.Text);
            String outPath = Path.Combine(@"c:\temp\", fileName);
            currentPage = 0;
            PDFConvertLib.effectImages(outPath, pageSize,
                delegate(string msg)
                {
                    var deleg = new MessageDelegate(CreatePdfMsg);
                    var ret = this.BeginInvoke(deleg, new object[] { msg });
                },
                delegate()
                {
                    var deleg = new EndDelegate(CreatePdfEndOperation);
                    var ret = this.BeginInvoke(deleg, new object[] { });
                },
                delegate(System.Drawing.Image img)
                {
                    var divX = (float)img.Width / leftPane.Width;
                    var divY = (float)img.Height / leftPane.Height;
                    return new System.Drawing.Rectangle(
                                (int)(divX * (leftRect.X - leftPane.Left)),
                                (int)(divY * (leftRect.Y - leftPane.Top)),
                                (int)(divX * (leftRect.Width)),
                                (int)(divY * (leftRect.Height))
                                );

                }
                , Gamma.Value / 10.0, numOfPages
            );
        }

        private void ExtractImageMsg(string str)
        {
            numOfPages++;
            Page.Text = "1 / " + numOfPages;
        }

        private void ExtractImageEndOperation()
        {
            updateImages();
            Refresh();
            CreatePdfData.Enabled = true;
        }

        private void Extract_Click(object sender, EventArgs e)
        {
            numOfPages = 0;
            currentPage = 1;
            Extract.Enabled = false;
            PDFConvertLib.ExtractImagesFromPDF(
                delegate(string msg){
                    var deleg = new MessageDelegate(ExtractImageMsg);
                    var ret = this.BeginInvoke(deleg, new object[] {msg});
                },
                delegate() {
                    var deleg = new EndDelegate(ExtractImageEndOperation);
                    var ret = this.BeginInvoke(deleg, new object[]{});
                },
                filePath.Text, PDFConvertLib.outputTempPath
            );
        }


        private void updateImages()
        {
            if (numOfPages > currentPage)
            {
                if (leftBmp != null) leftBmp.Dispose();
                string path = System.IO.Path.Combine(PDFConvertLib.outputTempPath, String.Format(PDFConvertLib.tempFileFormat, currentPage, 1));
                leftBmp = System.Drawing.Image.FromFile(path);
            }
            if (numOfPages > currentPage + 1)
            {
                if (rightBmp != null) rightBmp.Dispose();
                string path = System.IO.Path.Combine(PDFConvertLib.outputTempPath, String.Format(PDFConvertLib.tempFileFormat, currentPage + 1, 1));
                rightBmp = System.Drawing.Image.FromFile(path);
            }
        }

        private void previous_Click(object sender, EventArgs e)
        {
            if (currentPage > 2) currentPage -= 2;
            Page.Text = (currentPage + 1).ToString() + " / " + numOfPages;
            updateImages();
            this.Refresh();
        }

        private void next_Click(object sender, EventArgs e)
        {
            if (currentPage < numOfPages + 2) currentPage += 2;
            Page.Text = (currentPage + 1).ToString() + " / " + numOfPages;
            updateImages();
            this.Refresh();
        }

        private void MainForm_Paint(object sender, PaintEventArgs e)
        {
            if (numOfPages > currentPage && leftBmp != null)
            {
                var srcRect = new System.Drawing.Rectangle(0, 0, leftBmp.Width, leftBmp.Height);
                var destRect = new System.Drawing.Rectangle(leftPane.Left, leftPane.Top, leftPane.Width, leftPane.Height);
                e.Graphics.DrawImage(leftBmp, destRect, srcRect, GraphicsUnit.Pixel);
            }
            if (numOfPages > currentPage + 1 && rightBmp != null)
            {
                var srcRect = new System.Drawing.Rectangle(0, 0, rightBmp.Width, rightBmp.Height);
                var destRect = new System.Drawing.Rectangle(rightPane.Left, rightPane.Top, rightPane.Width, rightPane.Height);
                e.Graphics.DrawImage(rightBmp, destRect, srcRect, GraphicsUnit.Pixel);
            }
            if (leftRect.X != 0)
            {
                Graphics grfx = e.Graphics;
                Pen pen = new Pen(Color.Orange);
                SolidBrush brush = new SolidBrush(Color.Orange);
                grfx.DrawRectangle(pen, leftRect);
                rightRect.X = leftRect.X + rightPane.Left - leftPane.Left;
                rightRect.Y = leftRect.Y;
                rightRect.Width = leftRect.Width;
                rightRect.Height = leftRect.Height;
                grfx.DrawRectangle(pen, rightRect);
            }
        }

        private void MainForm_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.X > leftPane.Left && e.X < leftPane.Left + leftPane.Width / 2 &&
                e.Y > leftPane.Top && e.Y < leftPane.Top + leftPane.Height / 2)
            {
                leftRect.X = e.X; leftRect.Y = e.Y;
                if (leftRect.Width == 0 || leftRect.Height == 0)
                {
                    leftRect.Width = leftPane.Width - leftRect.X; leftRect.Height = leftPane.Height - leftRect.Y;
                }
            }
            if (e.X > leftPane.Right - leftPane.Width / 2 && e.X < leftPane.Right &&
              e.Y > leftPane.Bottom - leftPane.Height / 2 && e.Y < leftPane.Bottom)
            {
                leftRect.Width = e.X - leftRect.X; leftRect.Height = e.Y - leftRect.Y;
            }
            if (leftRect.Right == 0 && leftRect.Bottom == 0)
            {
                leftRect.X = leftPane.Left; leftRect.Y = leftPane.Top;
            }
            this.Refresh();
        }

        private Boolean isNumeric(String s)
        {
            Regex regex = new Regex(@"^[0-9]+$");
            return regex.IsMatch(s);
        }

        private void TargetScreenHeight_Validating(object sender, CancelEventArgs e)
        {
            if (!isNumeric(((TextBox)sender).Text)) 
            {
                e.Cancel = true;
            }
        }

        private void TargetScreenWidth_Validating(object sender, CancelEventArgs e)
        {
            if (!isNumeric(((TextBox)sender).Text))
            {
                e.Cancel = true;
            }
        }
    }
}
